use config::Config;
mod cats;
mod cfg;
use actix_files::Files;
use actix_web::{middleware, web, App, HttpServer};
use cats::db::{Db, Pool};
use r2d2_sqlite::{self, SqliteConnectionManager};

#[actix_web::main]
pub async fn main() -> std::io::Result<()> {
    let settings = Config::builder()
        .add_source(config::File::with_name("settings.yml"))
        .build()
        .unwrap();
    let app: cfg::AppConfig = settings.try_deserialize().unwrap();
    println!(
        "Запуск приложения в режиме {} {}:{}",
        app.debug, app.host, app.port
    );
    env_logger::init_from_env(env_logger::Env::new().default_filter_or("info"));
    log::info!("starting HTTP server at http://{}:{}", app.host, app.port);

    let manager = SqliteConnectionManager::file("cats.db");
    let pool = Pool::new(manager).unwrap();
    let db = Db::new(pool.clone()).await;
    db.init().await;

    HttpServer::new(move || {
        App::new()
            .app_data(web::Data::new(db.clone()))
            .wrap(middleware::Logger::default())
            .service(cats::get_cats)
            .service(cats::get_colors)
            .service(cats::add_cat)
            .service(cats::add_color)
            .service(Files::new("/", "./static/").index_file("index.html"))
    })
    .bind((app.host, app.port))?
    .run()
    .await
}
